# CMake macro for cleaning up the current directory
# F. Uhlig, 14 November 2020
# The macro removes all files from the subdirectory "data", all files matching
# the pattern all*.par, and all core dump files.
#
# Cleaning up the data directory as first step of a test suite provides a clean
# state and prevents accidental usage of files generated in previous tests.
# This is of particular relevance e.g., for ROOT parameter files, which will
# be updated instead of re-generated by successive test runs. 

Message("Remove all files from directory ${CMAKE_CURRENT_BINARY_DIR}/data")
file(GLOB _deletefiles
     LIST_DIRECTORIES false
     ${CMAKE_CURRENT_BINARY_DIR}/data/*
    )
foreach(file IN LISTS _deletefiles)
  file(REMOVE ${file})
endforeach()

Message("Remove all all*.par files from ${CMAKE_CURRENT_BINARY_DIR}")
file(GLOB _parfiles 
     LIST_DIRECTORIES false
     ${CMAKE_CURRENT_BINARY_DIR}/all*.par
    )
foreach(file IN LISTS _parfiles)
  file(REMOVE ${file})
endforeach()

Message("Remove all core dump files from ${CMAKE_CURRENT_BINARY_DIR}")
file(GLOB _dumpfiles 
     LIST_DIRECTORIES false
     ${CMAKE_CURRENT_BINARY_DIR}/core_dump_*
    )
foreach(file IN LISTS _dumpfiles)
  file(REMOVE ${file})
endforeach()
