/* Copyright (C) 2022 UGiessen, Giessen
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Florian Uhlig, Adrian Weber [committer]*/

#include <TFile.h>
#include <TGeoMatrix.h>

#include <map>
#include <string>

std::pair<std::string, TGeoHMatrix> AlignNode(std::string path, double shiftX, double shiftY, double shiftZ,
                                              double rotX, double rotY, double rotZ)
{

  TGeoHMatrix result;
  result.SetDx(shiftX);
  result.SetDy(shiftY);
  result.SetDz(shiftZ);
  result.RotateX(rotX);
  result.RotateY(rotY);
  result.RotateZ(rotZ);

  std::cout << "Alignment matrix for node " << path << " is: " << std::endl;
  result.Print();
  std::cout << std::endl;

  return std::pair<std::string, TGeoHMatrix>(path, result);
}


int create_alignment_2022_05_23_nickel()
{
  // Define the basic structure which needs to be filled with information
  // This structure is stored in the output file and later passed to the
  // FairRoot framework to do the (miss)alignment
  std::map<std::string, TGeoHMatrix> matrices;

  // ----------------  STS  ----------------------------//
  // Align full STS
  // matrices.insert(AlignNode("/cave_1/sts_v22c_mcbm_0", 0.275, -0.89, -.5, 0., 0., 0.));

  // Align individual STS Modules
  /* U0 L0 HL1 */
  /* M0 */ matrices.insert(
    AlignNode("/cave_1/sts_v22f_mcbm_0/Station01_1/Ladder09_1/HalfLadder09d_2/HalfLadder09d_Module03_1", +0.005, 0.000,
              0.005, 0., 0., 0.));
  /* M1 */ matrices.insert(
    AlignNode("/cave_1/sts_v22f_mcbm_0/Station01_1/Ladder09_1/HalfLadder09d_2/HalfLadder09d_Module03_2", +0.013, 0.000,
              -0.010, 0., 0., 0.));

  /* U0 L1 HL1 */
  /* M0 */ matrices.insert(
    AlignNode("/cave_1/sts_v22f_mcbm_0/Station01_1/Ladder09_2/HalfLadder09d_2/HalfLadder09d_Module03_1", -0.011, 0.000,
              +0.050, 0., 0., 0.));
  /* M1 */ matrices.insert(
    AlignNode("/cave_1/sts_v22f_mcbm_0/Station01_1/Ladder09_2/HalfLadder09d_2/HalfLadder09d_Module03_2", -0.019, 0.000,
              +0.050, 0., 0., 0.));

  /* U1 L0 HL1 */
  /* M0 */ matrices.insert(
    AlignNode("/cave_1/sts_v22f_mcbm_0/Station02_2/Ladder10_1/HalfLadder10d_2/HalfLadder10d_Module03_1", +0.030, -0.080,
              +0.610, 0., 0., 0.));
  /* M1 */ matrices.insert(
    AlignNode("/cave_1/sts_v22f_mcbm_0/Station02_2/Ladder10_1/HalfLadder10d_2/HalfLadder10d_Module04_2", 0.000, -0.080,
              +0.600, 0., 0., 0.));

  /* U1 L1 HL1 */
  /* M0 */ matrices.insert(
    AlignNode("/cave_1/sts_v22f_mcbm_0/Station02_2/Ladder12_2/HalfLadder12d_2/HalfLadder12d_Module03_1", -0.020, -0.020,
              -0.250, 0., 0., 0.));
  /* M1 */ matrices.insert(
    AlignNode("/cave_1/sts_v22f_mcbm_0/Station02_2/Ladder12_2/HalfLadder12d_2/HalfLadder12d_Module04_2", -0.010, -0.030,
              -0.200, 0., 0., 0.));

  /* U1 L2 HL1 */
  /* M0 */ matrices.insert(
    AlignNode("/cave_1/sts_v22f_mcbm_0/Station02_2/Ladder11_3/HalfLadder11d_2/HalfLadder11d_Module03_1", +0.020, -0.040,
              +0.610, 0., 0., 0.));
  /* M1 */ matrices.insert(
    AlignNode("/cave_1/sts_v22f_mcbm_0/Station02_2/Ladder11_3/HalfLadder11d_2/HalfLadder11d_Module03_2", +0.000, 0.000,
              0.000, 0., 0., 0.));
  /* M2 */ matrices.insert(
    AlignNode("/cave_1/sts_v22f_mcbm_0/Station02_2/Ladder11_3/HalfLadder11d_2/HalfLadder11d_Module03_3", -0.020, -0.040,
              +0.500, 0., 0., 0.));

  // ----------------  TRD  ----------------------------//
  /* TRD2D  */ matrices.insert(
    AlignNode("/cave_1/trd_v22h_mcbm_0/layer01_20101/module9_101001001", -2.0000, 0.000, 0.000, 0., 0., 0.));
  /* TRD1Dx */ matrices.insert(
    AlignNode("/cave_1/trd_v22h_mcbm_0/layer02_10202/module8_101002001", -2.8000, 0.000, -1.34, 0., 0., 0.));
  /* TRD1Dy */ matrices.insert(
    AlignNode("/cave_1/trd_v22h_mcbm_0/layer03_11303/module8_101303001", 0.0000, -3.0000, -1.34, 0., 0., 0.));


  // ----------------  TOF  ----------------------------//
  // Align tracking Tof
  // SM 0, RPC 0
  matrices.insert(AlignNode("/cave_1/tof_v21j_mcbm_0/tof_v21j_mcbmStand_1/module_0_0/gas_box_0/counter_0", -0.27, -1.29,
                            0.28 - 0.02, 0., 0.0, 0.));
  // SM 0, RPC 1
  matrices.insert(AlignNode("/cave_1/tof_v21j_mcbm_0/tof_v21j_mcbmStand_1/module_0_0/gas_box_0/counter_1", -0.25, -0.79,
                            -0.06 - 0.01, 0., 0.0, 0.));
  // SM 0, RPC 2
  matrices.insert(AlignNode("/cave_1/tof_v21j_mcbm_0/tof_v21j_mcbmStand_1/module_0_0/gas_box_0/counter_2", -0.27, -1.29,
                            0.28 - 0.02, 0., 0.0, 0.));
  // SM 0, RPC 3
  matrices.insert(AlignNode("/cave_1/tof_v21j_mcbm_0/tof_v21j_mcbmStand_1/module_0_0/gas_box_0/counter_3", -0.25, -0.79,
                            -0.06 - 0.01, 0., 0.0, 0.));
  // SM 0, RPC 4
  matrices.insert(AlignNode("/cave_1/tof_v21j_mcbm_0/tof_v21j_mcbmStand_1/module_0_0/gas_box_0/counter_4", -0.27, -1.29,
                            0.28 - 0.02, 0., 0.0, 0.));

  // SM 1, RPC 0
  matrices.insert(AlignNode("/cave_1/tof_v21j_mcbm_0/tof_v21j_mcbmStand_1/module_0_1/gas_box_0/counter_0", -0.27, -2.70,
                            -0.04 - 0.04, 0., 0.0, 0.));
  // SM 1, RPC 1
  matrices.insert(AlignNode("/cave_1/tof_v21j_mcbm_0/tof_v21j_mcbmStand_1/module_0_1/gas_box_0/counter_1", -0.21, -1.87,
                            -0.24 - 0.028, 0., 0.0, 0.));
  // SM 1, RPC 2
  matrices.insert(AlignNode("/cave_1/tof_v21j_mcbm_0/tof_v21j_mcbmStand_1/module_0_1/gas_box_0/counter_2", -0.27, -2.70,
                            -0.04 - 0.04, 0., 0.0, 0.));
  // SM 1, RPC 3
  matrices.insert(AlignNode("/cave_1/tof_v21j_mcbm_0/tof_v21j_mcbmStand_1/module_0_1/gas_box_0/counter_3", -0.21, -1.87,
                            -0.24 - 0.028, 0., 0.0, 0.));
  // SM 1, RPC 4
  matrices.insert(AlignNode("/cave_1/tof_v21j_mcbm_0/tof_v21j_mcbmStand_1/module_0_1/gas_box_0/counter_4", -0.27, -2.70,
                            -0.04 - 0.04, 0., 0.0, 0.));

  //   // SM 2, RPC 0
  //   matrices.insert(
  //     AlignNode("/cave_1/tof_v21j_mcbm_0/tof_v21j_mcbmStand_1/module_2_0/gas_box_0/counter_0",
  //             2.7+1.5, 0.0, -1.8-13, 0., 0.0, 0.));
  //   // SM 2, RPC 1
  //   matrices.insert(
  //     AlignNode("/cave_1/tof_v21j_mcbm_0/tof_v21j_mcbmStand_1/module_2_0/gas_box_0/counter_1",
  //             1.7, 0.0, -1.8-6.5, 0., 0.0, 0.));
  //   // SM 2, RPC 2
  //   matrices.insert(
  //     AlignNode("/cave_1/tof_v21j_mcbm_0/tof_v21j_mcbmStand_1/module_2_0/gas_box_0/counter_2",
  //             -0.7, 0.0, -1.8+0.0, 0., 0.0, 0.));
  //   // SM 2, RPC 3
  //   matrices.insert(
  //     AlignNode("/cave_1/tof_v21j_mcbm_0/tof_v21j_mcbmStand_1/module_2_0/gas_box_0/counter_3",
  //               1.7, 0.0, -1.8+6.5, 0., 0.0, 0.));
  //   // SM 2, RPC 4
  //   matrices.insert(
  //     AlignNode("/cave_1/tof_v21j_mcbm_0/tof_v21j_mcbmStand_1/module_2_0/gas_box_0/counter_4",
  //             2.7+1.5, 0.0, -1.8-13, 0., 0.0, 0.));

  // SM 3, RPC 0
  matrices.insert(AlignNode("/cave_1/tof_v21j_mcbm_0/tof_v21j_mcbmStand_1/module_0_2/gas_box_0/counter_0", -0.4,
                            -0.68 + 0.026, -1.8 - 0.01, 0., 0.0, 0.));
  // SM 3, RPC 1
  matrices.insert(AlignNode("/cave_1/tof_v21j_mcbm_0/tof_v21j_mcbmStand_1/module_0_2/gas_box_0/counter_1", -0.3,
                            -0.18 + 0.026, -1.8, 0., 0.0, 0.));
  // SM 3, RPC 2
  matrices.insert(AlignNode("/cave_1/tof_v21j_mcbm_0/tof_v21j_mcbmStand_1/module_0_2/gas_box_0/counter_2", -0.4,
                            -0.68 + 0.026, -1.8 - 0.01, 0., 0.0, 0.));
  // SM 3, RPC 3
  matrices.insert(AlignNode("/cave_1/tof_v21j_mcbm_0/tof_v21j_mcbmStand_1/module_0_2/gas_box_0/counter_3", -0.3,
                            -0.18 + 0.026, -1.8, 0., 0.0, 0.));
  // SM 3, RPC 4
  matrices.insert(AlignNode("/cave_1/tof_v21j_mcbm_0/tof_v21j_mcbmStand_1/module_0_2/gas_box_0/counter_4", -0.4,
                            -0.68 + 0.026, -1.8 - 0.01, 0., 0.0, 0.));


  // ---------------  RICH  ----------------------------//
  // Align full Rich
  //   matrices.insert(AlignNode("/cave_1/rich_v21c_mcbm_0/box_1", 0.0, 0.0, 0.0, 0., 0.0, 0.));


  // save matrices to disk
  TFile* misalignmentMatrixRootfile = new TFile("AlignmentMatrices_mcbm_beam_2022_05_23_nickel.root", "RECREATE");
  if (misalignmentMatrixRootfile->IsOpen()) {
    gDirectory->WriteObject(&matrices, "MisalignMatrices");
    misalignmentMatrixRootfile->Write();
    misalignmentMatrixRootfile->Close();
  }

  return 0;
}
