/* Copyright (C) 2023 GSI Helmholtzzentrum fuer Schwerionenforschung, Darmstadt
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Volker Friese [committer] */

/** @file Application.h
 ** @author Volker Friese <v.friese@gsi.de>
 ** @date 09.06.2023
 **/


#ifndef CBM_RECO_OFFLINE_APP_APPLICATION_H
#define CBM_RECO_OFFLINE_APP_APPLICATION 1

#include "ProgramOptions.h"
#include "Run.h"

namespace cbm::reco::offline
{

  class Application {


  public:
    /** @brief Standard constructor, initialises the application
     ** @param opt  **/
    explicit Application(ProgramOptions const& opt);

    /** @brief Copy constructor forbidden **/
    Application(const Application&) = delete;

    /** @brief Assignment operator forbidden **/
    void operator=(const Application&) = delete;

    /** @brief Destructor **/
    ~Application() = default;

    /** @brief Run the application **/
    void Exec();

  private:
    const std::string& OutputFile() const;
    const std::string& ParamFile() const;
    const std::string& SetupTag() const;
    const std::string& ConfigFile() const;


  private:
    ProgramOptions const& fOpt;  ///< Program options object
    Run fRun = {};               ///< Reconstruction run
  };

}  // namespace cbm::reco::offline

#endif /* CBM_RECO_OFFLINE_APP_APPLICATION */
