/* Copyright (C) 2022 GSI Helmholtzzentrum fuer Schwerionenforschung, Darmstadt
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Volker Friese [committer] */


#ifndef CBMTASKUNPACK_H
#define CBMTASKUNPACK_H 1

#include "CbmDefs.h"
#include "CbmDigiTimeslice.h"

#include <FairTask.h>

#include "tof/TofReadoutConfig.h"
#include "tof/UnpackTof.h"
#include "trd/TrdReadoutConfig.h"
#include "trd/UnpackTrd.h"
#include "trd2d/Trd2dReadoutConfig.h"
#include "trd2d/UnpackTrd2d.h"

#include <sstream>
#include <vector>

#include "EventBuilder.h"
#include "Unpack.h"
#include "bmon/BmonReadoutConfig.h"
#include "bmon/UnpackBmon.h"
#include "much/MuchReadoutConfig.h"
#include "much/UnpackMuch.h"
#include "sts/StsReadoutConfigLegacy.h"
#include "sts/UnpackSts.h"

class CbmDigiManager;
class CbmSourceTs;


/** @class CbmTaskUnpack
 ** @brief Task class for associating digis to events
 ** @author Volker Friese <v.friese@gsi.de>
 ** @since 15.11.2021
 **
 ** Creates objects of class CbmDigiEvent and fills them with digi objects,
 ** using the algorithm EventBuilder.
 **
 ** TOFO: The current implementation is for STS only and with a dummy trigger list
 ** just to establish the framework integration of algorithm and data interfaces.
 **/
class CbmTaskUnpack : public FairTask {


public:
  /** @brief Constructor **/
  CbmTaskUnpack();


  /** @brief Copy constructor (disabled) **/
  CbmTaskUnpack(const CbmTaskUnpack&) = delete;


  /** @brief Destructor **/
  virtual ~CbmTaskUnpack();


  /** @brief Task execution **/
  virtual void Exec(Option_t* opt);


  /** @brief Finish timeslice **/
  virtual void Finish();


  /** @brief Assignment operator (disabled) **/
  CbmTaskUnpack& operator=(const CbmTaskUnpack&) = delete;


private:  // methods
  /** @brief Task initialisation **/
  virtual InitStatus Init();


  /** @brief Initialisation of address maps for Trd **/
  cbm::algo::TrdReadoutConfig InitTrdReadoutConfig();


  /** @brief Initialisation of address maps for Trd2d **/
  cbm::algo::Trd2dReadoutConfig InitTrd2dReadoutConfig();

private:  // members
  CbmSourceTs* fSource = nullptr;

  /** @brief Unpacker algorithm **/
  cbm::algo::Unpack fUnpack;

  size_t fNumTs                = 0;
  size_t fNumMs                = 0;
  size_t fNumBytes             = 0;
  size_t fNumDigis             = 0;
  double fTime                 = 0.;
  CbmDigiTimeslice* fTimeslice = nullptr;  ///< Output data

  ClassDef(CbmTaskUnpack, 3);
};

#endif /* CBMTASKUNPACK_H */
